/*
 * Copyright (C) 2024 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU  Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
 *
 */

#ifndef MANAGEABLEWIDGET_H
#define MANAGEABLEWIDGET_H

#include <functional>

#include <QObject>
#include <QPoint>
#include <QString>
#include <QStringList>

#include "abstractconfigmonitor.h"

class ManageableObject;
using MouseButtonAction = std::function<void(ManageableObject&)>;

class ManageableObjectHost
{
public:
    virtual ~ManageableObjectHost();

    QStringList calcEffectiveSubmoduleNameList() const;
    void onModuleConfigChanged();

    bool shouldSubmoduleBeVisible(const QString& submoduleName) {
        const auto nameList = calcEffectiveSubmoduleNameList();
        return nameList.contains(submoduleName);
    }

protected:
    explicit ManageableObjectHost(const AbstractConfigMonitor& configMonitor);

    bool isSubmoduleNameKnown(const QString& submoduleName) const;

    void updateMouseButtonActions(const QString& moduleName);

private:
    virtual QStringList getKnownSubmoduleNameList() const { return {}; }
    virtual ManageableObject* getSubmoduleByName(const QString& submoduleName) {
        Q_UNUSED(submoduleName);

        return nullptr;
    }

    virtual QStringList getRawSubmoduleNameList() const { return getKnownSubmoduleNameList(); }
    virtual bool isSubmoduleValid(const QString& submoduleName) const {
        Q_UNUSED(submoduleName);

        return true;
    }

    void onSubmoduleConfigChanged(const QString& moduleName);

private:
    const AbstractConfigMonitor& configMonitor_;
    QMetaObject::Connection connection_;
};

class ManageableObject
{
public:
    virtual ~ManageableObject() = default;

    static const MouseButtonAction& getActivationAction() {
        static MouseButtonAction activationAction = [](ManageableObject& object) {
            object.doActivationAction();
        };

        return activationAction;
    }

    static const MouseButtonAction& getContextMenuAction() {
        static MouseButtonAction contextMenuAction = [](ManageableObject& object) {
            object.doContextMenuAction(object.eventPos);
        };

        return contextMenuAction;
    }

    static MouseButtonAction getMouseButtonActionByPolicy(const QString& actionPolicy) {
        if ("active" == actionPolicy) {
            return getActivationAction();
        }

        if ("active-menu" == actionPolicy) {
            return getContextMenuAction();
        }

        return nullptr;
    }

    virtual QString moduleName() const { return ""; }
    virtual void onConfigChanged() {}

    virtual void setLeftButtonClickedCallback(const MouseButtonAction& action) { Q_UNUSED(action); }
    virtual void setRightButtonClickedCallback(const MouseButtonAction& action) { Q_UNUSED(action); }
    virtual void restoreButtonClickedCallbacks() {}

    virtual void updateVisibility(bool visible) { Q_UNUSED(visible); }

    void updateMouseButtonActions();

protected:
    explicit ManageableObject(const AbstractConfigMonitor& configMonitor)
        : configMonitor_(configMonitor) {}

private:
    virtual void doActivationAction() {}
    virtual void doContextMenuAction(const QPoint& pos) { Q_UNUSED(pos); };

    void updateMouseLeftButtonAction();
    void updateMouseRightButtonAction();

protected:
    QPoint eventPos = {0, 0};

    const AbstractConfigMonitor& configMonitor_;
};

template <typename T>
class ManageableWidget : public ManageableObject
{
public:
    static void setLeftButtonClickedCallbackImpl(const MouseButtonAction& action)
    {
        leftButtonClickedCallback = action;
    }

    static void setRightButtonClickedCallbackImpl(const MouseButtonAction& action)
    {
        rightButtonClickedCallback = action;
    }

    static void restoreButtonClickedCallbacksImpl()
    {
        leftButtonClickedCallback = defaultLeftButtonClickedCallback;
        rightButtonClickedCallback = defaultRightButtonClickedCallback;
    }

    void setLeftButtonClickedCallback(const MouseButtonAction& action) override {
        setLeftButtonClickedCallbackImpl(action);
    }

    void setRightButtonClickedCallback(const MouseButtonAction& action) override {
        setRightButtonClickedCallbackImpl(action);
    }

    void restoreButtonClickedCallbacks() override {
        restoreButtonClickedCallbacksImpl();
    }

protected:
    explicit ManageableWidget(const AbstractConfigMonitor& configMonitor)
        : ManageableObject(configMonitor) {}

    ~ManageableWidget() override = default;

    void onLeftButtonClicked(const QPoint& pos)
    {
        doButtonClickedCallback(leftButtonClickedCallback, pos);
    }

    void onRightButtonClicked(const QPoint& pos)
    {
        doButtonClickedCallback(rightButtonClickedCallback, pos);
    }

private:
    void doButtonClickedCallback(const MouseButtonAction& buttonClickedCallback, const QPoint& pos)
    {
        if (buttonClickedCallback == nullptr)
        {
            return;
        }

        eventPos = pos;
        buttonClickedCallback(*this);
    }

protected:
    static MouseButtonAction leftButtonClickedCallback;
    static MouseButtonAction rightButtonClickedCallback;

    static const MouseButtonAction defaultLeftButtonClickedCallback;
    static const MouseButtonAction defaultRightButtonClickedCallback;
};

#endif
